//=======================================================================
//  TARGET PROJECT: PSM3(Prosessor Study Model3)
//  TARGET SYSTEM : Psm3Sys (PSM3 TEST SYSTEM)
//  FUNCTION      : Bus Splitter For DataBus
//
//                      Copyright (C) 2008 by Tomoyuki Inomoto
//=======================================================================
module BusSplit2(
    //======== SYSTEM ==============
    MemMapMode      ,

    //======= Master Bus ================
    MstrCRQ         ,
    MstrCAK         ,
    MstrBRST        ,
    MstrRW          ,
    MstrBE          ,
    MstrADR         ,
    MstrWD          ,
    MstrWDAK        ,
    MstrRD          ,
    MstrRDAK        ,

    //======= Flash Bus ================
    FlashCRQ        ,
    FlashCAK        ,
    FlashBRST       ,
    FlashRW         ,
    FlashBE         ,
    FlashADR        ,
    FlashWD         ,
    FlashWDAK       ,
    FlashRD         ,
    FlashRDAK       ,

    //======= Ddr Bus ================
    DdrCRQ          ,
    DdrCAK          ,
    DdrBRST         ,
    DdrRW           ,
    DdrBE           ,
    DdrADR          ,
    DdrWD           ,
    DdrWDAK         ,
    DdrRD           ,
    DdrRDAK         ,

    //======= Peri Bus ================
    PeriRQ          ,
    PeriAK          ,
    PeriRW          ,
    PeriBE          ,
    PeriADR         ,
    PeriWD          ,
    PeriRD
);
    //==================================================
    //======== port  ===================================
    //==================================================
    //======== SYSTEM ==============
    input           MemMapMode      ; // 0: AREA0 is DDR, 1:AREA0 is Flash

    //======= Master Bus ================
    input           MstrCRQ         ;
    output          MstrCAK         ;
    input           MstrBRST        ;
    input           MstrRW          ;
    input   [3:0]   MstrBE          ;
    input   [27:0]  MstrADR         ;
    input   [31:0]  MstrWD          ;
    output          MstrWDAK        ;
    output  [31:0]  MstrRD          ;
    output          MstrRDAK        ;

    //======= Flash Bus ================
    output          FlashCRQ        ;
    input           FlashCAK        ;
    output          FlashBRST       ;
    output          FlashRW         ;
    output  [3:0]   FlashBE         ;
    output  [25:0]  FlashADR        ;
    output  [31:0]  FlashWD         ;
    input           FlashWDAK       ;
    input   [31:0]  FlashRD         ;
    input           FlashRDAK       ;

    //======= Ddr Bus ================
    output          DdrCRQ          ;
    input           DdrCAK          ;
    output          DdrBRST         ;
    output          DdrRW           ;
    output  [3:0]   DdrBE           ;
    output  [25:0]  DdrADR          ;
    output  [31:0]  DdrWD           ;
    input           DdrWDAK         ;
    input   [31:0]  DdrRD           ;
    input           DdrRDAK         ;

    //======= Peri Bus ================
    output          PeriRQ          ;
    input           PeriAK          ;
    output          PeriRW          ;
    output  [3:0]   PeriBE          ;
    output  [15:0]  PeriADR         ;
    output  [31:0]  PeriWD          ;
    input   [31:0]  PeriRD          ;

    //==================================================
    //======== wire & reg ==============================
    //==================================================
    //wire            FlashSel        ;
    //wire            DdrSel          ;
    wire            PeriSel         ;
    //====================================================================
    //======== BODY ======================================================
    //====================================================================
    //======== PeriSel  =======================
    //assign  FlashSel = (MstrADR[27:26] == 2'b10) | ( MemMapMode & (MstrADR[27:26] == 2'b00));
    //assign  DdrSel   = (MstrADR[27:26] == 2'b01) | (~MemMapMode & (MstrADR[27:26] == 2'b00));
    assign  PeriSel  = (MstrADR[27:26] == 2'b11);

    //======= Master Bus ================
    //assign  MstrAK  = FlashCAK | DdrCAK | PeriAK;
    //assign  MstrWDAK = FlashWDAK | DdrWDAK | PeriAK;
    //assign  MstrRDAK = FlashRDAK | DdrRDAK | PeriAK;
    //assign  MstrRD  = (PeriSel)? PeriRD   :
    //                  (DdrSel)?   DdrRD   :
    //                              FlashRD ;

    assign  MstrCAK  = DdrCAK  | PeriAK;
    assign  MstrWDAK = DdrWDAK | PeriAK;
    assign  MstrRDAK = DdrRDAK | PeriAK;
    assign  MstrRD  = (PeriSel)? PeriRD : DdrRD ;

    //======= Flash Bus ================
    //assign  FlashCRQ  = MstrCRQ & MemMapMode;
    //assign  FlashBRST = MstrBRST;
    //assign  FlashRW  = MstrRW;
    //assign  FlashBE  = MstrBE;
    //assign  FlashADR = MstrADR[25:0];
    //assign  FlashWD  = MstrWD;

    assign  FlashCRQ  = 1'b0;
    assign  FlashBRST = 1'b0;
    assign  FlashRW  = 1'b0;
    assign  FlashBE  = 4'hF;
    assign  FlashADR = 26'h0;
    assign  FlashWD  = 32'h0;


    //======= Ddr Bus ================
    //assign  DdrCRQ  = MstrCRQ & DdrSel;
    assign  DdrCRQ  = MstrCRQ & ~PeriSel;
    assign  DdrBRST = MstrBRST;
    assign  DdrRW   = MstrRW;
    assign  DdrBE   = MstrBE;
    assign  DdrADR  = MstrADR[25:0];
    assign  DdrWD   = MstrWD;

    //======= Peri Bus ================
    assign  PeriRQ  = MstrCRQ & PeriSel;
    assign  PeriRW  = MstrRW;
    assign  PeriBE  = MstrBE;
    assign  PeriADR = MstrADR[15:0];
    assign  PeriWD  = MstrWD;

endmodule
