//============================================================
//  TARGET PROJECT: PSM3(Prosessor Study Model3)
//  TARGET SYSTEM : Psm3Sys (PSM3 TEST SYSTEM)
//  FUNCTION      : Gpio
//
//                      Copyright (C) 2008 by Tomoyuki Inomoto
//============================================================
`define     INPUT_PORT      2'h0
`define     OUTPUT_PORT     2'h1
`define     IO_PORT_MODE    2'h2
`define     IO_PORT_DATA    2'h3
module Gpio(
    //======== SYSTEM ==============
    Clk         ,
    RstN        ,

    //======== Peripheral Bus ======
    HostCS      ,
    HostRQ      ,
    HostRW      ,
    HostADR     ,
    HostWD      ,
    HostRD      ,

    //======== input Signal ======
    SlideSW     ,
    PushSW      ,
    RotCenter   ,
    RotA        ,
    RotB        ,

    //======== Output Signal ======
    LcdRW       ,
    LcdE        ,
    LcdRS       ,
    LcdDB       ,
    LED         ,
    //======== Input/Output Signal ======
    GpInOutMode ,
    GpInPut     ,
    GpOutPut
);
    //==================================================
    //======== port  ===================================
    //==================================================
    //======== SYSTEM ==============
    input           Clk         ;
    input           RstN        ;

    //======== Peripheral Bus ======
    input           HostCS      ;
    input           HostRQ      ;
    input           HostRW      ;
    input   [3:0]   HostADR     ;
    input   [31:0]  HostWD      ;
    output  [31:0]  HostRD      ;

    //======== input Signal ======
    input   [3:0]   SlideSW     ;
    input   [3:0]   PushSW      ;  //[3]:NOTH, [2]:SOUTH, [1]:EAST, [0]:WEST
    input           RotCenter   ;
    input           RotA        ;
    input           RotB        ;

    //======== Output Signal ======
    output          LcdRW       ;
    output          LcdE        ;
    output          LcdRS       ;
    output  [11:8]  LcdDB       ;
    output  [7:0]   LED         ;
    //======== Input/Output Signal ======
    output  [11:0]  GpInOutMode ;
    input   [11:0]  GpInPut     ;
    output  [11:0]  GpOutPut    ;
    //==================================================
    //======== wire & reg ==============================
    //==================================================
    reg     [3:0]   RegSlideSW  ;
    reg     [3:0]   RegPushSW   ;  //[3]:NOTH, [2]:SOUTH, [1]:EAST, [0]:WEST
    reg             RegRotCenter;
    reg             RegRotA     ;
    reg             RegRotB     ;

    reg             LcdRW       ;
    reg             LcdE        ;
    reg             LcdRS       ;
    reg     [11:8]  LcdDB       ;
    reg     [7:0]   LED         ;
    reg     [11:0]  GpInOutMode ;
    reg     [11:0]  RegGpInPut  ;
    reg     [11:0]  GpOutPut    ;
    wire            MyWe        ;
    wire            OutDataWe   ;
    //====================================================================
    //======== BODY ======================================================
    //====================================================================
    assign  MyWe = HostCS & HostRQ & HostRW;
    //======== INPUT_PORT ==============================
    always @(posedge Clk or negedge RstN) begin
        if(~RstN) begin
            RegSlideSW  <= 4'h0;
            RegPushSW   <= 4'h0;
            RegRotCenter<= 1'h0;
            RegRotA     <= 1'h0;
            RegRotB     <= 1'h0;
        end
        else begin
            RegSlideSW  <= SlideSW  ;
            RegPushSW   <= PushSW   ;
            RegRotCenter<= RotCenter;
            RegRotA     <= RotA     ;
            RegRotB     <= RotB     ;
        end
    end
    //======== OUTPUT_PORT ==============================
    always  @(posedge Clk or negedge RstN) begin
        if(~RstN) begin
            LcdRW   <= 1'h0;
            LcdE    <= 1'h0;
            LcdRS   <= 1'h0;
            LcdDB   <= 4'h0;
            LED     <= 8'h0;
        end
        else if(MyWe & (HostADR[3:2] == `OUTPUT_PORT)) begin
            LcdRW   <= HostWD[14];
            LcdE    <= HostWD[13];
            LcdRS   <= HostWD[12];
            LcdDB   <= HostWD[11:8];
            LED     <= HostWD[7:0];
        end
    end
    //======== IO_PORT_MODE ==============================
    always  @(posedge Clk or negedge RstN) begin
        if(~RstN) begin
            GpInOutMode <= 12'h0;
        end
        else if(MyWe & (HostADR[3:2] == `IO_PORT_MODE)) begin
            GpInOutMode <= HostWD[11:0];
        end
    end
    //======== IO_PORT_DATA ==============================
    assign  OutDataWe = MyWe & (HostADR[3:2] == `IO_PORT_DATA);
    always  @(posedge Clk or negedge RstN) begin
        if(~RstN)           GpOutPut <= 12'h0;
        else if(OutDataWe)  GpOutPut <= HostWD[11:0];
    end
    always  @(posedge Clk or negedge RstN) begin
        if(~RstN)   RegGpInPut <= 12'h0;
        else        RegGpInPut <= GpInPut;
    end

    //======== HostRD ==============================
    assign  HostRD = (HostADR[3:2] == `INPUT_PORT  )? {21'h0, RegRotB, RegRotA, RegRotCenter, RegPushSW, RegSlideSW} :
                     (HostADR[3:2] == `OUTPUT_PORT )? {17'h0, LcdRW, LcdE, LcdRS, LcdDB, LED} :
                     (HostADR[3:2] == `IO_PORT_MODE)? {20'h0, GpInOutMode} :
                                                      {20'h0, RegGpInPut } ;
endmodule
