//============================================================
//  TARGET PROJECT: PSM3(Prosessor Study Model3)
//  TARGET SYSTEM : Psm3Sys (PSM3 TEST SYSTEM)
//  FUNCTION      : Timer
//
//                      Copyright (C) 2008 by Tomoyuki Inomoto
//============================================================
`define     TIMER_CONTROL    2'h0
`define     TIMER_INTERVAL   2'h1
`define     TIMER_COUNTER    2'h2

module Timer(
    //======== SYSTEM ==============
    Clk         ,
    RstN        ,

    //======== Peripheral Bus ======
    HostCS      ,
    HostRQ      ,
    HostRW      ,
    HostADR     ,
    HostWD      ,
    HostRD      ,

    //======== Output Signal ======
    TimerIrq
);
    //==================================================
    //======== port  ===================================
    //==================================================
    //======== SYSTEM ==============
    input           Clk         ;
    input           RstN        ;

    //======== Peripheral Bus ======
    input           HostCS      ;
    input           HostRQ      ;
    input           HostRW      ;
    input   [3:0]   HostADR     ;
    input   [31:0]  HostWD      ;
    output  [31:0]  HostRD      ;

    //======== Output Signal ======
    output          TimerIrq    ;

    //==================================================
    //======== wire & reg ==============================
    //==================================================
    reg             Pulse1uS    ;
    reg     [6:0]   Counter1uS  ;
    reg             TimerIrq    ;
    reg             TimerRun    ;
    reg     [15:0]  Interval    ;
    reg     [15:0]  Counter     ;

    //====================================================================
    //======== BODY ======================================================
    //====================================================================
    //======== TimerRun ===================================
    //======== Interval ===================================
    always  @(posedge Clk or negedge RstN) begin
        if(~RstN) begin
            TimerRun <= 1'h0;
            Interval <= 16'h0;
        end
        else if(HostCS & HostRQ & HostRW) begin
            if(HostADR[3:2] == `TIMER_CONTROL ) TimerRun <= HostWD[0];
            if(HostADR[3:2] == `TIMER_INTERVAL) Interval <= HostWD[15:0];
        end
    end

    assign  HostRD = (HostADR[3:2] == `TIMER_CONTROL )? {31'h0, TimerRun}:
                     (HostADR[3:2] == `TIMER_INTERVAL)? {16'h0, Interval}:
                     (HostADR[3:2] == `TIMER_COUNTER )? {16'h0, Counter }: 32'h0;

    //======== Counter1uS ===================================
    always  @(posedge Clk or negedge RstN) begin
        if(~RstN) begin
            Counter1uS  <= 7'h0;
            Pulse1uS    <= 1'b0;
        end
        else if(TimerRun) begin
            //if(Counter1uS == 6'd49) begin
            if(Counter1uS == 7'd69) begin
                Counter1uS  <= 7'h0;
                Pulse1uS    <= 1'b1;
            end
            else begin
                Counter1uS  <= Counter1uS + 1'b1;
                Pulse1uS    <= 1'b0;
            end
        end
        else begin
            Counter1uS  <= 7'h0;
            Pulse1uS    <= 1'b0;
        end
    end

    //======== Counter ===================================
    always  @(posedge Clk or negedge RstN) begin
        if(~RstN) begin
            Counter     <= 16'h0;
            TimerIrq    <= 1'b0;
        end
        else if(TimerRun) begin
            if(Pulse1uS) begin
                if(Counter == Interval) begin
                    Counter     <= 16'h0;
                    TimerIrq    <= 1'b1;
                end
                else begin
                    TimerIrq    <= 1'b0;
                    Counter <= Counter + 1'b1;
                end
            end
            else begin
                TimerIrq    <= 1'b0;
            end
        end
        else begin
            Counter     <= 16'h0;
            TimerIrq    <= 1'b0;
        end
    end

endmodule
