//=======================================================================
//  TARGET PROJECT: PSM3(Prosessor Study Model3)
//  TARGET SYSTEM : *(PSM3 TARGET SYSTEM)
//  FUNCTION      : Psm3 Top Module of Processor Core
//
//                      Copyright (C) 2008-2010 by Tomoyuki Inomoto
//=======================================================================
// 20100327:UpDateAfter, PtrUpDate0, RfRdAdrB are removed
//         :Some signals about forwarding are added. (FwdData)
// 20100403:add Branch Prediction
//=======================================================================
module P3Core(
    //======== SYSTEM ========================
    Clk             ,
    RstN            ,
    Vector          ,
    PID             ,
    ICE             ,
    DCE             ,
    IRQ             ,
    BusLockRQ       ,
    BusLockAK       ,
    LockByDCash     ,

    //======= to/from Common Unit ======
    ComInB          ,
    ComInC          ,
    ComFunc         ,

    ComBSftRQ       ,
    ComBSftAK       ,
    ComBSftCmpl     ,
    ComBSftRes      ,

    ComDivRQ        ,
    ComDivAK        ,
    ComDivCmpl      ,
    ComDivRes       ,

    //======== INST BUS ================
    InstRQ          ,
    InstAK          ,
    InstADR         ,
    InstRD          ,
    InstBrInst      , // 20100403
    InstBrCountRd   , // 20100403
    InstBrCountWr   , // 20100403
    InstBrCountWe   , // 20100403

    //======== INST Cash ================
    ICashRe         ,
    ICashIndex      ,

    //======== DATA BUS ==================
    //DCashDrainRQ    , // 20100403
    DataRQ          ,
    DataAK          ,
    DataRW          ,
    DataBE          ,
    DataADR         ,
    DataWD          ,
    DataRD          ,

    //======== DATA Cash =================
    DCashRe         ,
    DCashArea       ,
    DCashIndex      ,

    //======== DEBUG================
    Break           ,
    DbgRq           ,
    DbgRegRq        ,
    DbgEna          ,
    DbgRegWe        ,
    DbgRegAdr       ,
    DbgWD           ,
    DbgRD           ,
    DecInstAK       ,
    Bubble          ,

    //======== MONITOR ================
    CoreGoAhead
);
    //============================================================
    //======== port  =============================================
    //============================================================
    //======== SYSTEM ========================
    input           Clk             ; // Clock
    input           RstN            ; // Reset (L-Active)
    input   [25:0]  Vector          ; // Vector of IRQ
    input           PID             ; // Processor ID
    output          ICE             ; // Inst Cash Enable
    output          DCE             ; // Data Cash Enable
    input           IRQ             ; // Interrupt Reqest
    output          BusLockRQ       ; // Bus Lock Reqest
    input           BusLockAK       ; // Bus Lock ACK
    input           LockByDCash     ; // InterLock Signal from DCash

    //======= to/from Common Unit =================
    output  [31:0]  ComInB          ; // PortB Data
    output  [31:0]  ComInC          ; // PortC Data
    output  [3:0]   ComFunc         ; // FUNCTION

    output          ComBSftRQ       ; // Common Barrel Shifter REQ
    input           ComBSftAK       ; // Common Barrel Shifter ACK
    input           ComBSftCmpl     ; // Common Barrel Shifter Complete
    input   [31:0]  ComBSftRes      ; // Result of Common Barrel Shifter

    output          ComDivRQ        ; // Common Divisor REQ
    input           ComDivAK        ; // Common Divisor ACK
    input           ComDivCmpl      ; // Common Divisor Complete
    input   [31:0]  ComDivRes       ; // Result of Common Divisor

    //======== INST BUS ================
    output          InstRQ          ; // INSTRUCTION BUS REQ
    input           InstAK          ; // INSTRUCTION BUS ACK
    output  [25:0]  InstADR         ; // INSTRUCTION BUS ADDRESS
    input   [31:0]  InstRD          ; // INSTRUCTION BUS READ DATA
    input           InstBrInst   ; // 20100403 indicate Inst is branch
    input   [1:0]   InstBrCountRd;// 20100403
    output  [1:0]   InstBrCountWr; // 20100403
    output          InstBrCountWe; // 20100403

    //======== INST Cash ================
    output          ICashRe         ; // Instruction Cash Read Enable
    //output  [10:2]  ICashIndex   ; // Instruction Cash Index (2KByte)
    output  [11:2]  ICashIndex   ; // Instruction Cash Index (4KByte)
    //output  [12:2]  ICashIndex   ; // Instruction Cash Index (8KByte)
    //output  [13:2]  ICashIndex   ; // Instruction Cash Index (16KByte)

    //======== DATA BUS ==================
    output          DataRQ          ; // DATA BUS REQ
    input           DataAK          ; // DATA BUS ACK
    output          DataRW          ; // DATA BUS READ/WRITE (1:WRITE)
    output  [3:0]   DataBE          ; // DATA BUS BYTE ENABLE
    output  [27:0]  DataADR         ; // DATA BUS ADDRESS
    output  [31:0]  DataWD          ; // DATA BUS WRITE DATA
    input   [31:0]  DataRD          ; // DATA BUS READ DATA

    //======== DATA Cash =================
    //output          DCashDrainRQ    ; // Data Cash Drain RQ (for Write Back Cash) //  20100403
    output          DCashRe         ; // Data Cash Read Enable
    output  [27:23] DCashArea       ; // Data Cash Accsess Area
    output  [11:2]  DCashIndex      ; // Data Cash Index (4KByte)

    //======== DEBUG================
    output          Break           ; // BREAK Instruction Detect Signal to Debugger
    input           DbgRq           ; // Debug Reqest from Debugger
    input           DbgRegRq        ; // Debug Reg Access Request from Debugger
    output          DbgEna          ; // Debug Enable to Debugger
    input           DbgRegWe        ; // Register Write Enable from Debugger
    input   [4:0]   DbgRegAdr       ; // Register Address from Debugger
    input   [31:0]  DbgWD           ; // Register Write Data from Debugger
    output  [31:0]  DbgRD           ; // Register Read  Data to   Debugger
    output          DecInstAK       ; // Ack of Instruction Reqest to DECU
    output          Bubble          ; // Bubble

    //======== MONITOR ================
    output          CoreGoAhead     ;

    //============================================================
    //======== wire & reg ========================================
    //============================================================
    //======== IFU to/form DECU ========
    wire    [25:0]  PC0             ; // Program Counter on IFU
    wire            DecInstRQ       ; // Instruction Reqest From DECU
    wire            DecInstAK       ; // Ack of Instruction Reqest to DECU
    wire    [31:0]  Inst            ; // Instruction Data
    wire            Break0          ; // Break0
    wire    [25:0]  JmpAdr          ; // Jump Address
    wire    [1:0]   NextPcSel       ; // 00:IncPC, 01:JmpAdr, 1*:Vector
    wire            PcWrited        ; // PC is OverWrited by Debuger

    //======== DECU to/from EXU =================
    wire            GoAhead         ; // Go Ahead Pipeline
    wire    [25:0]  PC1             ; // Program Counter at DECU
    wire    [4:0]   RfWrAdr0        ; // Register file Write Address
    wire            RfWe0           ; // Register file Write Enable
    wire    [31:0]  RfRdDataA       ; // PortA Read Data
    wire    [31:0]  RfRdDataB       ; // PortB Read Data
    wire    [31:0]  RfRdDataC       ; // PortC Read Data
    wire    [31:0]  RfRdDC          ; // GPR PortC Read Data
    wire    [15:0]  Imm16           ; // 16bit Immdiate
    wire            ImmSel          ; // Immdiate Select

    wire    [4:0]   RfWrAdrEx       ; // Register file Write Address
    wire            RfWeEx          ; // Register file Write Enable
    wire    [31:0]  RfWrDEx         ; // Register file Write Data

    wire    [3:0]   Func0           ; // ALU FUNCTION
    wire            AluAct0         ; // ALU ACTIVE
    wire            UpDateCV        ; // UpDate Carry & Over
    wire            UpDateSZ        ; // UpDate Sign & Zero
    wire            Over            ; // Flag
    wire            Carry           ; // Flag
    wire            Sign            ; // Flag
    wire            Zero            ; // Flag
    wire            AluWait         ; // ALU Wait
    wire    [31:0]  FwdData         ; // Data for forwarding // 20100327
    wire            FwdDataValid    ; // Data for forwarding valid // 20100327

    wire            DataRQ0         ; // DATA BUS REQEST (1:REQ)
    wire            DataRW0         ; // DATA BUS READ/WRITE (1:WRITE)
    wire    [2:0]   DataSIZE0       ; // DATA BUS DATA SIZE (000:WORD, 010:HALF, 011:Signed HALF, 100:BYTE, 101:Signed Half)
    //wire            DrainRQ0        ; // Data Cash Drain RQ (for Write Back Cash)  // 20100403 remove
    //wire            UpDateAfter     ; // Pointer UpDate After Access // 20100327 remove
    //wire            PtrUpDate0      ; // Pointer UpDate Flag // 20100327 remove
    wire            LsuWait         ; // LSU Wait
    //wire    [3:0]   RfRdAdrB        ; // Register file Read Address B // 20100327
    reg     [4:0]   DbgRegAdrBuf    ; // Register Address from Debugger
    wire            BrPredict0      ; // Branch Prediction act // 20100403

    //============================================================
    //======== BODY       ========================================
    //============================================================
    //=========================================
    //======== etc. ===========================
    //=========================================
    assign  CoreGoAhead = GoAhead;

    always  @(posedge Clk or negedge RstN) begin
        if(~RstN)   DbgRegAdrBuf <= 4'h0;
        else        DbgRegAdrBuf <= DbgRegAdr;
    end

    //=========================================
    //======== IFU ============================
    //=========================================
    P3Ifu P3Ifu(
        .Clk             ( Clk             ),
        .RstN            ( RstN            ),
        .Vector          ( Vector          ),
        .PC0             ( PC0             ),
        .DecInstRQ       ( DecInstRQ       ),
        .DecInstAK       ( DecInstAK       ),
        .Inst            ( Inst            ),
        .Break0          ( Break0          ),
        .JmpAdr          ( JmpAdr          ),
        .BrPredict0      ( BrPredict0      ), // 20100403 add
        .NextPcSel       ( NextPcSel       ),
        .PcWrited        ( PcWrited        ),
        .InstRQ          ( InstRQ          ),
        .InstAK          ( InstAK          ),
        .InstADR         ( InstADR         ),
        .InstRD          ( InstRD          ),
        .InstBrInst      ( InstBrInst      ), // 20100403
        .InstBrCountRd   ( InstBrCountRd   ), // 20100403
        .InstBrCountWr   ( InstBrCountWr   ), // 20100403
        .InstBrCountWe   ( InstBrCountWe   ), // 20100403
        .ICashRe         ( ICashRe         ),
        .ICashIndex      ( ICashIndex      ),
        .DbgRegRq        ( DbgRegRq        ),
        .DbgRegWe        ( DbgRegWe        ),
        .DbgRegAdr       ( DbgRegAdrBuf    ),
        .DbgWD           ( DbgWD           )
    );

    //=========================================
    //======== DECU ===========================
    //=========================================
    P3Decu P3Decu(
        .Clk             ( Clk             ),
        .RstN            ( RstN            ),
        .PID             ( PID             ),
        .ICE             ( ICE             ),
        .DCE             ( DCE             ),
        .IRQ             ( IRQ             ),
        .BusLockRQ       ( BusLockRQ       ),
        .BusLockAK       ( BusLockAK       ),
        .LockByDCash     ( LockByDCash     ),
        .DecInstRQ       ( DecInstRQ       ),
        .DecInstAK       ( DecInstAK       ),
        .PC0             ( PC0             ),
        .Inst            ( Inst            ),
        .Break0          ( Break0          ),
        .NextPcSel       ( NextPcSel       ),
        .PcWrited        ( PcWrited        ),
        .JmpAdr          ( JmpAdr          ),
        .BrPredict0      ( BrPredict0      ), // 20100403 add
        .GoAhead         ( GoAhead         ),
        .PC1             ( PC1             ),
        .RfWrAdr0        ( RfWrAdr0        ),
        //.RfRdAdrB        ( RfRdAdrB        ), // 20100327
        .RfWe0           ( RfWe0           ),
        .RfRdDataA       ( RfRdDataA       ),
        .RfRdDataB       ( RfRdDataB       ),
        .RfRdDataC       ( RfRdDataC       ),
        .RfRdDC          ( RfRdDC          ),
        .Imm16           ( Imm16           ),
        .ImmSel          ( ImmSel          ),
        .RfWrAdrEx       ( RfWrAdrEx       ),
        .RfWeEx          ( RfWeEx          ),
        .RfWrDEx         ( RfWrDEx         ),
        .Func0           ( Func0           ),
        .AluAct0         ( AluAct0         ),
        .UpDateCV        ( UpDateCV        ),
        .UpDateSZ        ( UpDateSZ        ),
        .Over            ( Over            ),
        .Carry           ( Carry           ),
        .Sign            ( Sign            ),
        .Zero            ( Zero            ),
        .AluWait         ( AluWait         ),
        .FwdData         ( FwdData         ), // 20100327 add
        .FwdDataValid    ( FwdDataValid    ), // 20100327 add
        .DataRQ0         ( DataRQ0         ),
        .DataRW0         ( DataRW0         ),
        .DataSIZE0       ( DataSIZE0       ),
        //.DrainRQ0        ( DrainRQ0        ), // 20100403 remove
        //.UpDateAfter     ( UpDateAfter     ), // 20100327 remove
        //.PtrUpDate0      ( PtrUpDate0      ), // 20100327 remove
        .LsuWait         ( LsuWait         ),
        .Break           ( Break           ),
        .DbgRq           ( DbgRq           ),
        .DbgRegRq        ( DbgRegRq        ),
        .DbgEna          ( DbgEna          ),
        .DbgRegWe        ( DbgRegWe        ),
        .DbgRegAdr       ( DbgRegAdrBuf    ),
        .DbgWD           ( DbgWD           ),
        .DbgRD           ( DbgRD           ),
        .Bubble          ( Bubble          )
    );

    //=========================================
    //======== EXU ============================
    //=========================================
    P3Exu P3Exu(
        .Clk             ( Clk             ),
        .RstN            ( RstN            ),
        .GoAhead         ( GoAhead         ),
        .PC1             ( PC1             ),
        .RfWrAdr0        ( RfWrAdr0        ),
        //.RfRdAdrB        ( RfRdAdrB        ), // 20100327 remove
        .RfWe0           ( RfWe0           ),
        .RfRdDataA       ( RfRdDataA       ),
        .RfRdDataB       ( RfRdDataB       ),
        .RfRdDataC       ( RfRdDataC       ),
        .RfRdDC          ( RfRdDC          ),
        .Imm16           ( Imm16           ),
        .ImmSel          ( ImmSel          ),
        .RfWrAdrEx       ( RfWrAdrEx       ),
        .RfWeEx          ( RfWeEx          ),
        .RfWrDEx         ( RfWrDEx         ),
        .Func0           ( Func0           ),
        .AluAct0         ( AluAct0         ),
        .UpDateCV        ( UpDateCV        ),
        .UpDateSZ        ( UpDateSZ        ),
        .Over            ( Over            ),
        .Carry           ( Carry           ),
        .Sign            ( Sign            ),
        .Zero            ( Zero            ),
        .AluWait         ( AluWait         ),
        .FwdData         ( FwdData         ), // 20100327 add
        .FwdDataValid    ( FwdDataValid    ), // 20100327 add
        .DataRQ0         ( DataRQ0         ),
        .DataRW0         ( DataRW0         ),
        .DataSIZE0       ( DataSIZE0       ),
        //.DrainRQ0        ( DrainRQ0        ),  // 20100403 remove
        //.UpDateAfter     ( UpDateAfter     ),  // 20100327 remove
        //.PtrUpDate0      ( PtrUpDate0      ),  // 20100327 remove
        .LsuWait         ( LsuWait         ),

        .ComInB          ( ComInB          ),
        .ComInC          ( ComInC          ),
        .ComFunc         ( ComFunc         ),

        .ComBSftRQ       ( ComBSftRQ       ),
        .ComBSftAK       ( ComBSftAK       ),
        .ComBSftCmpl     ( ComBSftCmpl     ),
        .ComBSftRes      ( ComBSftRes      ),

        .ComDivRQ        ( ComDivRQ        ),
        .ComDivAK        ( ComDivAK        ),
        .ComDivCmpl      ( ComDivCmpl      ),
        .ComDivRes       ( ComDivRes       ),

        .DataRQ          ( DataRQ          ),
        .DataAK          ( DataAK          ),
        .DataRW          ( DataRW          ),
        .DataBE          ( DataBE          ),
        .DataADR         ( DataADR         ),
        .DataWD          ( DataWD          ),
        .DataRD          ( DataRD          ),
        //.DCashDrainRQ    ( DCashDrainRQ    ),  // 20100403
        .DCashRe         ( DCashRe         ),
        .DCashArea       ( DCashArea       ),
        .DCashIndex      ( DCashIndex      ),
        .DbgRegRq        ( DbgRegRq        ),
        .DbgWD           ( DbgWD           )
    );

endmodule
