//=======================================================================
//  TARGET PROJECT: PSM3(Prosessor Study Model3)
//  TARGET SYSTEM : *(PSM3 TARGET SYSTEM)
//  FUNCTION      : Top Module of Psm3(Core + L1-Cash)
//
//                      Copyright (C) 2008 by Tomoyuki Inomoto
//=======================================================================
// 20100403:add Branch Prediction
//=======================================================================
module P3Top(
    //======== SYSTEM ========================
    Clk             ,
    RstN            ,
    Vector          ,
    PID             ,
    IRQ             ,
    BusLockRQ       ,
    BusLockAK       ,

    //======= to/from Common Unit ======
    ComInB          ,
    ComInC          ,
    ComFunc         ,

    ComBSftRQ       ,
    ComBSftAK       ,
    ComBSftCmpl     ,
    ComBSftRes      ,

    ComDivRQ        ,
    ComDivAK        ,
    ComDivCmpl      ,
    ComDivRes       ,

    //======== INST BUS ================
    InstCRQ         ,
    InstCAK         ,
    InstBRST        ,
    InstADR         ,
    InstRD          ,
    InstRDAK        ,

    //======== DATA BUS ==================
    DataCRQ         ,
    DataCAK         ,
    DataBRST        ,
    DataRW          ,
    DataBE          ,
    DataADR         ,
    DataWD          ,
    DataWDAK        ,
    DataRD          ,
    DataRDAK        ,

    //======== DEBUG================
    CashClr         ,
    Break           ,
    DbgRq           ,
    DbgRegRq        ,
    DbgEna          ,
    DbgRegWe        ,
    DbgRegAdr       ,
    DbgWD           ,
    DbgRD           ,
    DecInstAK       ,
    Bubble          ,

    //======== MONITOR ================
    CoreGoAhead     ,
    CoreInstRQ      ,
    CoreInstAK      ,
    CoreInstHit     ,
    CoreDataRQ      ,
    CoreDataAK      ,
    CoreDataHit
);
    //============================================================
    //======== port  =============================================
    //============================================================
    //======== SYSTEM ========================
    input           Clk             ; // Clock
    input           RstN            ; // Reset (L-Active)
    input   [25:0]  Vector          ; // Vector of IRQ
    input           PID             ; // Processor ID
    input           IRQ             ; // Interrupt Reqest
    output          BusLockRQ       ; // Bus Lock Reqest
    input           BusLockAK       ; // Bus Lock ACK

    //======= to/from Common Unit =================
    output  [31:0]  ComInB          ; // PortB Data
    output  [31:0]  ComInC          ; // PortC Data
    output  [3:0]   ComFunc         ; // FUNCTION

    output          ComBSftRQ       ; // ComBSft REQ
    input           ComBSftAK       ; // ComBSft ACK
    input           ComBSftCmpl     ; // ComBSft Complete
    input   [31:0]  ComBSftRes      ; // Result of ComBSft

    output          ComDivRQ        ; // Common Divisor REQ
    input           ComDivAK        ; // Common Divisor ACK
    input           ComDivCmpl      ; // Common Divisor Complete
    input   [31:0]  ComDivRes       ; // Result of Common Divisor

    //======== INST BUS ================
    output          InstCRQ         ; // INSTRUCTION BUS COMMAND REQ
    input           InstCAK         ; // INSTRUCTION BUS COMMAND ACK
    output          InstBRST        ; // INSTRUCTION BUS SINGLE/BURST
    output  [25:0]  InstADR         ; // INSTRUCTION BUS ADDRESS
    input   [31:0]  InstRD          ; // INSTRUCTION BUS READ DATA
    input           InstRDAK        ; // INSTRUCTION BUS READ DATA ACK

    //======== DATA BUS ==================
    output          DataCRQ         ; // DATA BUS COMMAND REQ
    input           DataCAK         ; // DATA BUS COMMAND ACK
    output          DataBRST        ; // DATA BUS SINGLE/BURST
    output          DataRW          ; // DATA BUS READ/WRITE (1:WRITE)
    output  [3:0]   DataBE          ; // DATA BUS BYTE ENABLE
    output  [27:0]  DataADR         ; // DATA BUS ADDRESS
    output  [31:0]  DataWD          ; // DATA BUS WRITE DATA
    input           DataWDAK        ; // DATA BUS WRITE DATA ACK
    input   [31:0]  DataRD          ; // DATA BUS READ DATA
    input           DataRDAK        ; // DATA BUS READ DATA ACK

    //======== DEBUG================
    input           CashClr         ; // Cash Clear Signal frim Debugger
    output          Break           ; // BREAK Instruction Detect Signal to Debugger
    input           DbgRq           ; // Debug Reqest from Debugger
    input           DbgRegRq        ; // Debug Reg Access Request from Debugger
    output          DbgEna          ; // Debug Enable to Debugger
    input           DbgRegWe        ; // Register Write Enable from Debugger
    input   [4:0]   DbgRegAdr       ; // Register Address from Debugger
    input   [31:0]  DbgWD           ; // Register Write Data from Debugger
    output  [31:0]  DbgRD           ; // Register Read  Data to   Debugger
    output          DecInstAK       ; // Ack of Instruction Reqest to DECU
    output          Bubble          ; // Bubble

    //======== MONITOR ================
    output          CoreGoAhead     ;
    output          CoreInstRQ      ;
    output          CoreInstAK      ;
    output          CoreInstHit     ;
    output          CoreDataRQ      ;
    output          CoreDataAK      ;
    output          CoreDataHit     ;
    //============================================================
    //======== wire & reg ========================================
    //============================================================
    //======== SYSTEM ========================
    wire            ICE             ; // Inst Cash Enable
    wire            DCE             ; // Data Cash Enable

    //======== INTERNAL INST BUS ================
    //wire            CoreInstRQ      ; // CORE INSTRUCTION BUS REQ
    //wire            CoreInstAK      ; // CORE INSTRUCTION BUS ACK
    wire    [25:0]  CoreInstADR     ; // CORE INSTRUCTION BUS ADDRESS
    wire    [31:0]  CoreInstRD      ; // CORE INSTRUCTION BUS READ DATA
    wire            CoreBrInst      ; // 20100403 indicate Inst is branch
    wire    [1:0]   CoreBrCountIC2IF;// 20100403
    wire    [1:0]   CoreBrCountIF2IC; // 20100403
    wire            CoreBrCountWe   ; // 20100403

    
    //======== INST Cash ================
    wire            ICashRe         ; // Instruction Cash Read Enable
    wire    [11:2]  ICashIndex   ; // Instruction Cash Index (4KByte)

    //======== INTERNAL DATA BUS ==================
    //wire            CoreDataRQ      ; // CORE DATA BUS REQ
    //wire            CoreDataAK      ; // CORE DATA BUS ACK
    wire            CoreDataRW      ; // CORE DATA BUS READ/WRITE (1:WRITE)
    wire    [3:0]   CoreDataBE      ; // CORE DATA BUS BYTE ENABLE
    wire    [27:0]  CoreDataADR     ; // CORE DATA BUS ADDRESS
    wire    [31:0]  CoreDataWD      ; // CORE DATA BUS WRITE DATA
    wire    [31:0]  CoreDataRD      ; // CORE DATA BUS READ DATA

    //======== DATA Cash =================
    //wire            DCashDrainRQ    ; // Data Cash Drain RQ (for Write Back Cash) // 20100403
    wire            DCashRe         ; // Data Cash Read Enable
    wire    [27:23] DCashArea       ; // Data Cash Accsess Area
    wire    [11:2]  DCashIndex      ; // Data Cash Index (4KByte)
    wire            LockByDCash     ; // InterLock Signal from DCash

    //======== DEBUG =================
    reg             CashClr1d       ; // Buffer of CashClr

    //============================================================
    //======== BODY       ========================================
    //============================================================
    //=========================================
    //======== PSM3 Core ======================
    //=========================================
    P3Core P3Core(
        .Clk             ( Clk             ),
        .RstN            ( RstN            ),
        .Vector          ( Vector          ),
        .PID             ( PID             ),
        .ICE             ( ICE             ),
        .DCE             ( DCE             ),
        .IRQ             ( IRQ             ),
        .BusLockRQ       ( BusLockRQ       ),
        .BusLockAK       ( BusLockAK       ),
        .LockByDCash     ( LockByDCash     ),

        .ComInB          ( ComInB          ),
        .ComInC          ( ComInC          ),
        .ComFunc         ( ComFunc         ),

        .ComBSftRQ       ( ComBSftRQ       ),
        .ComBSftAK       ( ComBSftAK       ),
        .ComBSftCmpl     ( ComBSftCmpl     ),
        .ComBSftRes      ( ComBSftRes      ),

        .ComDivRQ        ( ComDivRQ        ),
        .ComDivAK        ( ComDivAK        ),
        .ComDivCmpl      ( ComDivCmpl      ),
        .ComDivRes       ( ComDivRes       ),

        .InstRQ          ( CoreInstRQ      ),
        .InstAK          ( CoreInstAK      ),
        .InstADR         ( CoreInstADR     ),
        .InstRD          ( CoreInstRD      ),
        .InstBrInst      ( CoreBrInst      ), // 20100403
        .InstBrCountRd   ( CoreBrCountIC2IF), // 20100403
        .InstBrCountWr   ( CoreBrCountIF2IC), // 20100403
        .InstBrCountWe   ( CoreBrCountWe   ), // 20100403
        .ICashRe         ( ICashRe         ),
        .ICashIndex      ( ICashIndex      ),
        .DataRQ          ( CoreDataRQ      ),
        .DataAK          ( CoreDataAK      ),
        .DataRW          ( CoreDataRW      ),
        .DataBE          ( CoreDataBE      ),
        .DataADR         ( CoreDataADR     ),
        .DataWD          ( CoreDataWD      ),
        .DataRD          ( CoreDataRD      ),
        //.DCashDrainRQ    ( DCashDrainRQ    ), // 20100403
        .DCashRe         ( DCashRe         ),
        .DCashArea       ( DCashArea       ),
        .DCashIndex      ( DCashIndex      ),
        .Break           ( Break           ),
        .DbgRq           ( DbgRq           ),
        .DbgRegRq        ( DbgRegRq        ),
        .DbgEna          ( DbgEna          ),
        .DbgRegWe        ( DbgRegWe        ),
        .DbgRegAdr       ( DbgRegAdr       ),
        .DbgWD           ( DbgWD           ),
        .DbgRD           ( DbgRD           ),
        .DecInstAK       ( DecInstAK       ),
        .Bubble          ( Bubble          ),
        .CoreGoAhead     ( CoreGoAhead     )
    );
    //=========================================
    //======== CashClr1d  =====================
    //=========================================
    always  @(posedge Clk or negedge RstN) begin
        if(~RstN)   CashClr1d <= 2'h0;
        else        CashClr1d <= CashClr & DbgRq;
    end

    //=========================================
    //======== L1-Instruction Cash ============
    //=========================================
    P3ICash4K32b4Brst P3ICash( // 4KByte
        .Clk             ( Clk             ),
        .RstN            ( RstN            ),
        .ICE             ( ICE & ~CashClr1d),
        .MstrRQ          ( CoreInstRQ      ),
        .MstrAK          ( CoreInstAK      ),
        .MstrADR         ( CoreInstADR     ),
        .MstrRD          ( CoreInstRD      ),
        .MstrBrInst      ( CoreBrInst      ), // 20100403
        .MstrBrCountOut  ( CoreBrCountIC2IF), // 20100403
        .MstrBrCountIn   ( CoreBrCountIF2IC), // 20100403
        .MstrBrCountWe   ( CoreBrCountWe   ), // 20100403
        .MstrICRe        ( ICashRe         ),
        .MstrICIndex     ( ICashIndex      ),
        .ExCRQ           ( InstCRQ         ),
        .ExCAK           ( InstCAK         ),
        .ExBRST          ( InstBRST        ),
        .ExADR           ( InstADR         ),
        .ExRD            ( InstRD          ),
        .ExRDAK          ( InstRDAK        ),
        .CoreInstHit     ( CoreInstHit     )
    );

    //=========================================
    //======== L1-Data Cash ===================
    //=========================================
    P3DCash4K32bWB P3DCash( // 4KByte
        .Clk             ( Clk             ),
        .RstN            ( RstN            ),
        .DCE             ( DCE & ~CashClr1d),
        .MstrRQ          ( CoreDataRQ      ),
        .MstrAK          ( CoreDataAK      ),
        .MstrRW          ( CoreDataRW      ),
        .MstrBE          ( CoreDataBE      ),
        .MstrADR         ( CoreDataADR     ),
        .MstrWD          ( CoreDataWD      ),
        .MstrRD          ( CoreDataRD      ),
        //.MstrDCDrainRQ   ( DCashDrainRQ    ), // 20100403
        .MstrDCDrainRQ   ( 1'b0            ), // 20100403
        .MstrDCRe        ( DCashRe         ),
        .MstrDCArea      ( DCashArea       ),
        .MstrDCIndex     ( DCashIndex      ),
        .MstrLock        ( LockByDCash     ),
        .ExCRQ           ( DataCRQ         ),
        .ExCAK           ( DataCAK         ),
        .ExBRST          ( DataBRST        ),
        .ExRW            ( DataRW          ),
        .ExBE            ( DataBE          ),
        .ExADR           ( DataADR         ),
        .ExWD            ( DataWD          ),
        .ExWDAK          ( DataWDAK        ),
        .ExRD            ( DataRD          ),
        .ExRDAK          ( DataRDAK        ),
        .CoreDataHit     ( CoreDataHit     )
    );

endmodule
